@@ -19,11 +19,25 @@
 
 #include <string.h>
 #include <stdio.h>
-#include <gmp.h>
 
-#include "src/compiled.h"
 #include "floattypes.h"
 
+#include <mpfi.h>
+
+/****************************************************************
+ * mpfi's are stored as follows:
+ * +-----------+-----------------------------------------+---------------------+
+ * | TYPE_MPFI |             __mpfi_struct               |    __mp_limb_t[]    |
+ * |           | __mpfr_struct left         right        | limbl ... limbr ... |
+ * |           | prec exp sign mant   prec exp sign mant |                     |
+ * +-----------+-----------------------------------------+---------------------+
+ *                               \____________________\____^         ^
+ *                                                     \____________/
+ * it is assumed that the left and right mpfr's are allocated with the
+ * same precision
+ ****************************************************************/
+#define MPFI_OBJ(obj) ((mpfi_ptr) (ADDR_OBJ(obj)+1))
+
 #define LMANTISSA_MPFI(p) ((mp_limb_t *) (p+1))
 #define RMANTISSA_MPFI(p) (LMANTISSA_MPFI(p)+(mpfi_get_prec(p)+GMP_NUMB_BITS-1)/GMP_NUMB_BITS)
 
@@ -596,7 +610,7 @@ static Obj MPFI_STRING(Obj self, Obj s, Obj prec)
     n = GET_LEN_STRING(s)*1000 / 301;
 
   Obj g = NEW_MPFI(INT_INTOBJ(prec));
-  mpfi_set_str(MPFI_OBJ(g), (char *)CHARS_STRING(s), 10);
+  mpfi_set_str(MPFI_OBJ(g), CONST_CSTR_STRING(s), 10);
   return g;
 }
 
